/*
   This subprogram determines the MPI_rank of the nearest neighbors 
   in a N-dimensional regular grid.
*/

#include <stdio.h>
#include "mpi.h"


/*@

  RGX_Find_Task_Grid_Ngbrs - Determine the task IDs of the nearest neighbors of
                             local task grid

  Unmodified Input Variables :
. Comm - Input Communicator of all the tasks involved.
. Ndim - Dimensionality of the Grid (Integer).

  Modified Output Variables :
. Ngbrs - Integer Array, of size 2 x Ndim, of the task ID of the local task 
          grid's nearest neighbors.  It is implicitly assumed to be declared 
          like Ngbrs[ Ndim ][ 2 ].
. returned value - returns MPI_SUCCESS if completed successfully.
.n

.N RGX_fortran
.N RGX_minimal_set_of_variables
.N RGX_supported_datatype
.N RGX_yes_included

@*/
int RGX_Find_Task_Grid_Ngbrs(
                              MPI_Comm  Comm,
                              int       Ndim,

                              int       Ngbrs[]
                            )
/*
    In Fortran :
       Ngbrs(1,1) = Ngbr_left, Ngbrs(2,1) = Ngbr_right
       Ngbrs(1,2) = Ngbr_top , Ngbrs(2,2) = Ngbr_bottom
       Ngbrs(1,3) = Ngbr_in  , Ngbrs(2,3) = Ngbr_out
    In C:
       Ngbrs[ 0 ] = Ngbr_left, Ngbrs[ 1 ] = Ngbr_right
       Ngbrs[ 2 ] = Ngbr_top , Ngbrs[ 3 ] = Ngbr_bottom
       Ngbrs[ 4 ] = Ngbr_in  , Ngbrs[ 5 ] = Ngbr_out
*/
{

   /*  Local Variables:  */
   int  ierr, ii;

   for ( ii = 0; ii < Ndim; ii++ )
   {
      ierr = MPI_Cart_shift( Comm, ii, 1, &Ngbrs[ 2*ii ], &Ngbrs[ 2*ii+1 ] );
      if ( ierr != MPI_SUCCESS )
      {
         fprintf( stderr, __FILE__": MPI_Cart_shift() fails at idir = %d "
                          " with ierr = %d\n", ii, ierr );
         return ierr;
      }
   }
   return MPI_SUCCESS;

}



#ifdef FortranNoUnderscore
#   define rgx_find_task_grid_ngbrs__  rgx_find_task_grid_ngbrs
#endif

#ifdef FortranSingleUnderscore
#   define rgx_find_task_grid_ngbrs__  rgx_find_task_grid_ngbrs_
#endif

/*  Fortran Interface for RGX_Find_Task_Grid_Ngbrs()  */
void rgx_find_task_grid_ngbrs__(
                                 MPI_Comm *Comm,
                                 int      *Ndim,

                                 int      *Ngbrs,
                                 int      *ierr
                               )
{
   *ierr = RGX_Find_Task_Grid_Ngbrs( *Comm, *Ndim, Ngbrs );
}
