#include <stdio.h>
#if defined( sgi )
#include <alloca.h>
#else
#include <stdlib.h>
#endif

#include "mpi.h"


/*
   This file contains a routine for producing a decomposition of a 1-d array
   when given a number of processors.  It may be used in "direct" product
   decomposition.  The values returned assume a "global" domain in [1:n]
*/
int Decomp1d( int n, int size, int rank, int *s, int *e )
{
    int nlocal, deficit;

    nlocal      = n / size;
    *s  = rank * nlocal + 1;
    deficit     = n % size;
    *s  = *s + ((rank < deficit) ? rank : deficit);
    if (rank < deficit) nlocal++;
    *e      = *s + nlocal - 1;
    if (*e > n || rank == size-1) *e = n;
    return MPI_SUCCESS;
}


/*
    This subprogram determines the end-points coordinates of the local grid 
    resides in the local task grid

    RGX_Set_Glb_Grid_Ppty( Comm, Ndim, Glb_Grid_sz, stride_width ) needs to be
    called BEFORE RGX_Find_Task_Grid_endpts()
*/


/*@
  RGX_Find_Task_Grid_endpts - Determine local task grid's end-points coodinates

  Unmodified Input Variables :
. Comm - Input Communicator of all the tasks involved.
. Ndim - Dimensionality of the Grid (Integer).
. Glb_Grid_sz - Integer Array, of size Ndim, specifies the maximum grid size 
                of the global grid.  Glb_Grid_sz[ ii ] is the maximum grid size
                in the dimension ii, where the minimum is assumed to be 1.

  Modified Output Variables :
. task_Grid_endpts - Integer Array, of size 2 x Ndim, of the two end-points'
                     coordinates of the LOCAL task grid.  It is implicitly
                     assumed to be declared like task_Grid_endpts[ 2 ][ Ndim ].
. returned value - returns MPI_SUCCESS if completed successfully
.n

.N RGX_fortran
.N RGX_minimal_set_of_variables
.N RGX_supported_datatype
.N RGX_yes_included

@*/

int RGX_Find_Task_Grid_endpts(
                               MPI_Comm    Comm,
                               int         Ndim,
                               int         Glb_Grid_sz[],

                               int         task_Grid_endpts[]
                             )
{

   /*  Local Variables:  */
   int *N_Grids_1dim;
   int *periods;
   int *coords;
   int  ierr, ii;


   /*  Allocate memory for local variables:  */

   N_Grids_1dim = ( int * ) alloca( Ndim * sizeof( int ) );
   periods      = ( int * ) alloca( Ndim * sizeof( int ) );
   coords       = ( int * ) alloca( Ndim * sizeof( int ) );

   ierr = MPI_Cart_get( Comm, Ndim, N_Grids_1dim, periods, coords );
   if ( ierr !=  MPI_SUCCESS )
   {
      fprintf( stderr, __FILE__": MPI_Cart_get() fails with ierr = %d\n",
               ierr );
      return ierr;
   }

   for ( ii = 0; ii < Ndim; ii++ )
      Decomp1d( Glb_Grid_sz[ ii ], N_Grids_1dim[ ii ], coords[ ii ],
                &task_Grid_endpts[ ii ], &task_Grid_endpts[ ii + Ndim ] );

   return MPI_SUCCESS;
}



#ifdef FortranNoUnderscore
#   define rgx_find_task_grid_endpts__  rgx_find_task_grid_endpts
#endif

#ifdef FortranSingleUnderscore
#   define rgx_find_task_grid_endpts__  rgx_find_task_grid_endpts_
#endif

/*   Fortran Interface for RGX_Find_Task_Grid_endpts()   */
void rgx_find_task_grid_endpts__(
                                  MPI_Comm   *Comm,
                                  int        *Ndim,
                                  int        *Glb_Grid_sz,

                                  int        *task_Grid_endpts,
                                  int        *ierr
                                )
{
   *ierr = RGX_Find_Task_Grid_endpts( *Comm, *Ndim, Glb_Grid_sz,
                                      task_Grid_endpts );
}
